-- Lemken_Solitair
-- Specialization for Lemken_Solitair mod
--
-- @author  Tobias F. (John Deere 6930, 1. LS-Modschmiede)
-- @date LastEdit 02/10/09

Lemken_Solitair = {};

function Lemken_Solitair.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function Lemken_Solitair:load(xmlFile) 
	local jointTypeStr = getXMLString(xmlFile, "vehicle.attacherJoint#jointType");
	if jointTypeStr ~= nil then
		local jointType = Vehicle.jointTypeNameToInt[jointTypeStr];
		if jointType == nil then
			Vehicle.registerJointType(jointTypeStr);
			print("register Jointtype: " .. jointTypeStr);
			jointType = Vehicle.jointTypeNameToInt[jointTypeStr];
		end;
		if jointType ~= nil then
			self.attacherJoint.jointType = jointType;			
		end;
	end;
    self.SupportComp = self.components[2].node --Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.Support#Compindex"));
	self.Marker = {};
	local NodeMarkerBack = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.Marker.Back#index"));
    if NodeMarkerBack ~= nil then
        self.Marker.Back = {};
        self.Marker.Back.node = NodeMarkerBack;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.Marker.Back#minRot"));
        self.Marker.Back.minRot = {};
        self.Marker.Back.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.Marker.Back.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.Marker.Back.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.Marker.Back#maxRot"));
        self.Marker.Back.maxRot = {};
        self.Marker.Back.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.Marker.Back.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.Marker.Back.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
        self.Marker.Back.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.Marker.Back#rotTime"), 2)*1000;
        self.Marker.Back.isDown = false;
    end;
	self.UsageScale = Utils.getNoNil(getXMLString(xmlFile, "vehicle.UsageScale#value"), 1);

    local cultivatorSound = getXMLString(xmlFile, "vehicle.cultivatorSound#file");
    if cultivatorSound ~= nil and cultivatorSound ~= "" then
        cultivatorSound = Utils.getFilename(cultivatorSound, self.baseDirectory);
        self.cultivatorSound = createSample("cultivatorSound");
        loadSample(self.cultivatorSound, cultivatorSound, false);
        self.cultivatorSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.cultivatorSound#pitchOffset"), 0);
        self.cultivatorSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.cultivatorSound#volume"), 1.0);
        self.cultivatorSoundEnabled = false;
    end;
    local AirseederInSound = getXMLString(xmlFile, "vehicle.AirseederInSound#file");
    if AirseederInSound ~= nil and AirseederInSound ~= "" then
        AirseederInSound = Utils.getFilename(AirseederInSound, self.baseDirectory);
        self.AirseederInSound = createSample("AirseederInSound");
        loadSample(self.AirseederInSound, AirseederInSound, false);
        self.AirseederInSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.AirseederInSound#pitchOffset"), 0);
        self.AirseederInSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.AirseederInSound#volume"), 1.0);
        self.AirseederInSoundEnabled = false;
    end;
    local AirseederOffSound = getXMLString(xmlFile, "vehicle.AirseederOffSound#file");
    if AirseederOffSound ~= nil and AirseederOffSound ~= "" then
        AirseederOffSound = Utils.getFilename(AirseederOffSound, self.baseDirectory);
        self.AirseederOffSound = createSample("AirseederOffSound");
        loadSample(self.AirseederOffSound, AirseederOffSound, false);
        self.AirseederOffSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.AirseederOffSound#pitchOffset"), 0);
        self.AirseederOffSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.AirseederOffSound#volume"), 1.0);
        self.AirseederOffSoundEnabled = false;
    end;
    local AirseederSound = getXMLString(xmlFile, "vehicle.AirseederSound#file");
    if AirseederSound ~= nil and AirseederSound ~= "" then
        AirseederSound = Utils.getFilename(AirseederSound, self.baseDirectory);
        self.AirseederSound = createSample("AirseederSound");
        loadSample(self.AirseederSound, AirseederSound, false);
        self.AirseederSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.AirseederSound#pitchOffset"), 0);
        self.AirseederSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.AirseederSound#volume"), 1.0);
        self.AirseederSoundEnabled = false;
    end;	
	self.playOffSound =  false;
	setMass(self.components[1].node, 0.35);
	setMass(self.components[2].node, 10000);
end;



function Lemken_Solitair:delete()
    if self.cultivatorSound ~= nil then
        delete(self.cultivatorSound);
    end;
    if self.AirseederInSound ~= nil then
        delete(self.AirseederInSound);
    end;
    if self.AirseederOffSound ~= nil then
        delete(self.AirseederOffSound);
    end;
    if self.AirseederSound ~= nil then
        delete(self.AirseederSound);
    end;
end;

function Lemken_Solitair:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Lemken_Solitair:keyEvent(unicode, sym, modifier, isDown)
end;

function Lemken_Solitair:update(dt)
	if self.attacherVehicle ~= nil then
		setMass(self.components[1].node, 0.35)
--	    Lemken_Solitair<->Lemken_Attacher Controls
		if self.attacherVehicle.isLemkenAttacher then
			local LemkenAttacher = self.attacherVehicle;
			if LemkenAttacher.aiBackMarker ~= nil then
				LemkenAttacher.aiBackMarker = self.aiBackMarker;
			end;
			
			if self:getIsActiveForInput() then
		        if InputBinding.hasEvent(InputBinding.S_BACKMARKER) then
		            self.Marker.Back.isDown = not self.Marker.Back.isDown;
		        end;
		    end;
			
			if self.Marker.Back ~= nil then
				if not self.Marker.Back.isDown then
					local x, y, z = getRotation(self.Marker.Back.node);
					local rot = {x,y,z};
					local newRot = Utils.getMovedLimitedValues(rot, self.Marker.Back.maxRot, self.Marker.Back.minRot, 3, self.Marker.Back.rotTime, dt,true);
					setRotation(self.Marker.Back.node, unpack(newRot));
				else
					local x, y, z = getRotation(self.Marker.Back.node);
					local rot = {x,y,z};
					local newRot = Utils.getMovedLimitedValues(rot, self.Marker.Back.maxRot, self.Marker.Back.minRot, 3, self.Marker.Back.rotTime, dt,false);
					setRotation(self.Marker.Back.node, unpack(newRot));
					if LemkenAttacher.Marker.areActivated then
						if LemkenAttacher.contactReportNodes[LemkenAttacher.GroundContactReportNode].hasGroundContact then
							if self.cuttingAreas[2] ~= nil then
								local x,y,z = getWorldTranslation(self.cuttingAreas[2].start);
								local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[2].width);
								local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[2].height);
								Utils.updatePloughArea(x, z, x1, z1, x2, z2);
							end; 
							if self.cuttingAreas[3] ~= nil then
								local x,y,z = getWorldTranslation(self.cuttingAreas[3].start);
								local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[3].width);
								local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[3].height);
								Utils.updatePloughArea(x, z, x1, z1, x2, z2);
							end; 
						end;
					end;
				end;
			end;	
			
			if not LemkenAttacher.Marker.areActivated then
	            self.Marker.Back.isDown = false;
	   		end;
			if LemkenAttacher.isTurnedOn then
				if self.AirseederInSound ~= nil and not self.AirseederInSoundEnabled and self:getIsActiveForSound() then
					playSample(self.AirseederInSound, 1, self.AirseederInSoundVolume, 0);
					setSamplePitch(self.AirseederInSound, self.AirseederInSoundPitchOffset);
					self.AirseederInSoundEnabled = true;
					self.playOffSound =  true;
				end;
				if self.AirseederSound ~= nil and not self.AirseederSoundEnabled and self:getIsActiveForSound() then
					playSample(self.AirseederSound, 0, self.AirseederSoundVolume, 0);
					setSamplePitch(self.AirseederSound, self.AirseederSoundPitchOffset);
					self.AirseederSoundEnabled = true;
				end;
				if self.AirseederOffSoundEnabled then
					stopSample(self.AirseederOffSound);
					self.AirseederOffSoundEnabled = false;
				end;	
				if self.movingDirection > 0 and LemkenAttacher.contactReportNodes[LemkenAttacher.GroundContactReportNode].hasGroundContact then
					if self.cultivatorSound ~= nil and not self.cultivatorSoundEnabled and self:getIsActiveForSound() then
						if self.lastSpeed*3600 > 3 then
							playSample(self.cultivatorSound, 0, self.cultivatorSoundVolume, 0);
							setSamplePitch(self.cultivatorSound, self.cultivatorSoundPitchOffset);
							self.cultivatorSoundEnabled = true;
						end;
					end;
					---- amazone front tank -----
					local attacher = self.attacherVehicle;
					while attacher.attacherVehicle~= nil do
						attacher=attacher.attacherVehicle;
					end;
					local fruittype = SeedConsomation(FruitUtil.FRUITTYPE_UNKNOWN,0,attacher.attachedImplements);
					if fruittype ~= FruitUtil.FRUITTYPE_UNKNOWN	then
					---- amazone front tank /-----
						if self.cuttingAreas[1] ~= nil then
							local cArea = self.cuttingAreas[1];
							local x,y,z = getWorldTranslation(cArea.start);
							local x1,y1,z1 = getWorldTranslation(cArea.width);
							local x2,y2,z2 = getWorldTranslation(cArea.height);
							local area = Utils.updateSowingArea(fruittype, x, z, x1, z1, x2, z2);
							---- amazone front tank -----
							local fruitDesc = FruitUtil.fruitIndexToDesc[fruittype];
							---- amazone front tank /-----
							local pixelToQm = 2048 / 8192 * 2048 / 8192; -- 8192px are mapped to 2048m
							local qm = area*pixelToQm;
							local ha = qm/10000;
							local usage = fruitDesc.seedUsagePerQm*qm*self.UsageScale;
							---- amazone front tank -----
							SeedConsomation(self.attacherVehicle,usage,attacher.attachedImplements);
							---- amazone front tank /-----
							g_currentMission.missionStats.seedUsageTotal = g_currentMission.missionStats.seedUsageTotal + usage;
							g_currentMission.missionStats.seedUsageSession = g_currentMission.missionStats.seedUsageSession + usage;
							g_currentMission.missionStats.hectaresSeededTotal = g_currentMission.missionStats.hectaresSeededTotal + ha;
							g_currentMission.missionStats.hectaresSeededSession = g_currentMission.missionStats.hectaresSeededSession + ha;
						end;
						g_currentMission.missionStats.seedingDurationTotal = g_currentMission.missionStats.seedingDurationTotal + dt/(1000*60);
						g_currentMission.missionStats.seedingDurationSession = g_currentMission.missionStats.seedingDurationSession + dt/(1000*60);
					end;
				else
					if self.cultivatorSoundEnabled then
						stopSample(self.cultivatorSound);
						self.cultivatorSoundEnabled = false;
					end;
				end;
			else
				if self.AirseederOffSound ~= nil and self.playOffSound and not self.AirseederOffSoundEnabled and self:getIsActiveForSound() then
					playSample(self.AirseederOffSound, 1, self.AirseederOffSoundVolume, 0);
					setSamplePitch(self.AirseederOffSound, self.AirseederOffSoundPitchOffset);
					self.AirseederOffSoundEnabled = true;
					self.playOffSound = false;
				end;

				if self.AirseederSoundEnabled then
					stopSample(self.AirseederSound);
					self.AirseederSoundEnabled = false;
				end;	
				if self.AirseederInSoundEnabled then
					stopSample(self.AirseederInSound);
					self.AirseederInSoundEnabled = false;
				end;					
			end;
			if self.cultivatorSoundEnabled then
				if self.lastSpeed*3600 < 3 then
					stopSample(self.cultivatorSound);
					self.cultivatorSoundEnabled = false;
				end;
			end;
		end;
	end;
	

	
	if self.attacherVehicle ~= nil then
		setRigidBodyType(self.components[2].node, "NoRigidBody");
        local index1 = 0;
        local index2 = 1;
        local jointIndexStr = '0>1';

        local jointNode = Utils.indexToObject(self.components, jointIndexStr);
        if jointNode ~= nil and jointNode ~= 0 then
        	local x, y, z = getWorldTranslation(jointNode)
        	setTranslation(self.components[2].node, x, y, z)
        	local x, y, z = getWorldRotation(jointNode)
        	setRotation(self.components[2].node, x, y, z)
		end;
	end;
	if self.SupportCompAttach then
		local index1 = 0;
		local index2 = 1;
		local jointIndexStr = '0>1';
		local jointNode = Utils.indexToObject(self.components, jointIndexStr);
		if jointNode ~= nil and jointNode ~= 0 then
			local x, y, z = getWorldTranslation(jointNode)
			setTranslation(self.components[2].node, x, y, z)
			local x, y, z = getWorldRotation(jointNode)
			setRotation(self.components[2].node, x, y, z)

			local constr = JointConstructor:new();
			constr:setActors(self.components[index1+1].node, self.components[index2+1].node);
			constr:setJointTransforms(jointNode, jointNode);

			local x, y, z = Utils.getVectorFromString('0 0 0');
			local rotLimits = {};
			rotLimits[1] = math.rad(Utils.getNoNil(x, 0));
			rotLimits[2] = math.rad(Utils.getNoNil(y, 0));
			rotLimits[3] = math.rad(Utils.getNoNil(z, 0));

			local x, y, z = Utils.getVectorFromString('0 0 0');
			local transLimits = {};
			transLimits[1] = Utils.getNoNil(x, 0);
			transLimits[2] = Utils.getNoNil(y, 0);
			transLimits[3] = Utils.getNoNil(z, 0);

			for i=1, 3 do
				local rotLimit = rotLimits[i];
				if rotLimit >= 0 then
					constr:setRotationLimit(i-1, -rotLimit, rotLimit);
				end;

				local transLimit = transLimits[i];
				if transLimit >= 0 then
					constr:setTranslationLimit(i-1, true, -transLimit, transLimit);
				else
					constr:setTranslationLimit(i-1, false, 0, 0);
				end;
			end;
			local jointDesc = {};
			jointDesc.componentIndices = {index1+1, index2+1};
			jointDesc.jointNode = jointNode;
			jointDesc.jointIndex = constr:finalize();
			table.insert(self.componentJoints, jointDesc);
			self.SupportCompAttach = false;
		end;
	end;
end;

function Lemken_Solitair:draw()
	if self.attacherVehicle ~= nil then
		if self.attacherVehicle.Marker ~= nil then
			if self.attacherVehicle.Marker.areActivated then
				if self.Marker.Back.isDown then
					g_currentMission:addHelpButtonText(g_i18n:getText("MARKER_BACK"), InputBinding.S_BACKMARKER);
				else
					g_currentMission:addHelpButtonText(g_i18n:getText("MARKER_BACK"), InputBinding.S_BACKMARKER);
				end;
			end;
		end;
	end;
end;

function Lemken_Solitair:onAttach(attacherVehicle)
	self.SupportCompAttach = false;
	setRigidBodyType(self.SupportComp, "NoRigidBody");
	setVisibility(self.SupportComp, false);
	--setMass(self.components[1].node, 0.35)
	--setMass(self.components[2].node, 0.001)
end;

function Lemken_Solitair:onLeave()
	self:onDeactivateSounds(self);
end;

function Lemken_Solitair:onDeactivateSounds()
	if self.AirseederSoundEnabled then
		stopSample(self.AirseederSound);
		self.AirseederSoundEnabled = false;
	end;	
	if self.AirseederOffSoundEnabled then
		stopSample(self.AirseederOffSound);
		self.AirseederOffSoundEnabled = false;
	end;	
	if self.AirseederInSoundEnabled then
		stopSample(self.AirseederInSound);
		self.AirseederInSoundEnabled = false;
	end;	
    if self.cultivatorSoundEnabled then
        stopSample(self.cultivatorSound);
        self.cultivatorSoundEnabled = false;
    end;
end;

function Lemken_Solitair:onDetach()
	self:onDeactivateSounds(self);
	self.SupportCompAttach = true;
    setRigidBodyType(self.SupportComp, "Dynamic");
	setVisibility(self.SupportComp, true);
	
	--setMass(self.components[1].node, 0.35)
	--setMass(self.components[2].node, 10)
end;
---- amazone front tank -----
function SeedConsomation(fruittype,usage,objectAttachedImplements)
	for k, implement in pairs(objectAttachedImplements) do
		local object = implement.object;
		fruittype = SeedConsomation(foundSeed,usage,object.attachedImplements)
		if object.fillLevel ~= nil and object.capacity ~= nil then
			if object.fillLevel > 0 then
				object.fillLevel = math.max(object.fillLevel-usage,0)
				fruittype = object.currentFillType;
				break;
			end;
		end;		
	end;
	return fruittype;
end;
---- amazone front tank /-----